-- The Stapler class.  All the core functionality is in this class.
-- To use:
--
--         s = Stapler:new()              -- create a new instance
--         s.functions = {...}            -- load functions
--         results = s:process_text(text) -- run
--
-- What Stapler does to the text depends entirely on the functions
-- that get loaded.

module(..., package.seeall)

Stapler = {

   -- Create a new parser and make it an instance of this class
   new = function (self)
      obj = { functions = {}}
      setmetatable(obj, self)
      self.__index = self
      return obj
   end,

   -- Process a chunk of text, looking for [foo ...] markup
   -- Returns a list of items (what the items are depends on 
   -- the functions loaded into the parser
   process_text = function (self, text)
      local results = {}
      if string.match(text, "%b[]") then 
	 local before, bracketed, after = string.match(text, 
						    "^([^\[]*)(%b[])(.*)")
	 results[1] = before
	 -- interpret the braketed part
	 results[2] = self:process_markup(bracketed)
	 -- process the rest recursively
	 for i, result in ipairs(self:process_text(after)) do
	    table.insert(results, result)
	 end
	 return results
      else
	 return {text}
      end
   end,

   -- Process a [foo ...] string
   process_markup = function(self, markup)
      local tag, rest = string.match(markup, "^%[(.-)%s+(.*)%]$")
      -- make a new objected, metatabled to self, to pass down in recursion
      -- this way downstream code can see and change self but 
      -- but the changes will disappear when the recursion pops
      local newself = {}
      setmetatable(newself, self)
      self.__index = self
      newself.functions = {}
      setmetatable(newself.functions, self.functions)
      self.functions.__index = self.functions
      -- now do the (potentially) recursive call
      return self.functions[tag](newself, rest)
   end,
}




parsers = {}

-- Splits text into the first "word" (up to the first space) and the rest
function parsers.vrest (text) 
   return {string.match(text, "^(.-)%s+(.*)$")}
end

-- Splits text into the first two words and the rest
function parsers.vvrest (text)
   local v1, rest = parsers.vrest(text)
   local v2, rest = parsers.vrest(rest)
   return {v1, v2, rest}
end

-- Splits text into the first three words and the rest
function parsers.vvvrest (text)
   local v1, v2, rest = parsers.vvrest(text)
   local v3, rest = parsers.vrest(rest)
   return {v1, v2, v3, rest}
end


