--- HTML Tags -----------------------------------------------------

require"stapler"

-------------------------------------------------------------------
-- A utility function that creates a function for an html tags.
-- Accepts a table of arguments, expecting the following values:
--
--    args.tag    - the html tag that will be generated (required)
--    args.parser - a function for parsing the text (optional)
--    args.attrs  - names of attributes (must be on less than the
--                  number of the results returned by parser (optional)
--    args.pre    - a function to be called before recursing (optional)  
--    args.fn     - a function to run on the children produced by 
--                  process_text (optional)
-------------------------------------------------------------------

function make_html_function(args)
   return function(s, text)
      local buffer = "<"..args.tag 
      if parser then
	 local parts = parser(text)
	 for i, attr in ipairs(args.attrs) do
	    buffer = buffer.." "..attr.."='"..(parts[i] or "").."'"
	 end
	 text = parts[#parts]
      end
      if args.pre then
	 args.pre(s)
      end
      children = s:process_text(text)
      if args.fn then
	 children = args.fn(children)
      end
      return buffer..">"..table.concat(children, "").."</"..args.tag..">"
   end   
end

function make_html_function2(args)
   return function(s, text)
      local t = {tag=args.tag, args={}}
      if parser then
	 local parts = parser(text)
	 for i, attr in ipairs(args.attrs) do
	    t.args[attr] = parts[i]
	 end
	 text = parts[#parts]
      end
      if args.pre then
	 args.pre(s)
      end
      children = s:process_text(text)
      if args.fn then
	 children = args.fn(children)
      end
      for i,child in ipairs(children) do
	 t[i] = child
      end
      return t
   end   
end


-- makes a bunch of functions for handing html tags and returns them
-- as a table
function make_html_functions() 

   if MAKE_TREE == true then
      make_html_function = make_html_function2
   end
   
   local html = {}
   -- functions without arguments
   for i, tag in ipairs{
      'b', 'i', 'em', 'strong', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6',
      'p',
   } do
      html[tag] = make_html_function{tag=tag}
   end
   -- other functions
   html.a = make_html_function{
      tag="a",
      parser=stapler.parsers.vrest, 
      args={'href'}
   }
   html.ul = make_html_function{
      tag="ul",
      pre = function(s)
	       s.functions['-'] = make_html_function{tag="li"}
	    end
   }
   html.div = make_html_function{
      tag="div",
      parser=stapler.parsers.vrest,
      args={'id'}
   }
   return html
end


MAKE_TREE = true


function print_html_tree (tree) 
   if type(tree) == "string" then
      io.write(tree)
      return
   end
   if tree.tag then
      buffer = "<"..tree.tag
      for k,v in pairs(tree.args) do
	 buffer= buffer.." "..k.."='"..v.."'"
      end
      buffer = buffer .. ">"
      io.write(buffer)
   end
   for i,child in ipairs(tree) do
      print_html_tree(child)
   end
   if tree.tag then
      io.write("</"..tree.tag..">")
   end
end

function test_html()
      
   local source = [=[
          [h1 The Title]
	  Test file for using 
	  [: Stapler] 
          with [: HTML].
          [em [p some [b bold text]]]
	  [a http://www.freewisdom.org/  Some website]
	  [ul
	     [- one]
	     [- two]
	     [- three]
	  ]
]=]


   s = stapler.Stapler:new()

   s.indent = ""
   s.functions = make_html_functions()

   -- let's also add a function that will treat [: as wikilinks
   s.functions[":"] = function(s, text)
			 return string.format("<a href='%s%s'>%s</a>",
					      "http://localhost/wiki/",
					      text, text)
					   end
   local r = s:process_text(source)

   if MAKE_TREE == true then
      print("---------")
      print_html_tree(r)
      print("---------")
   else
      print(table.concat(r))
   end
end

test_html()